package com.application.GUI.PopUpWindows;

import javafx.geometry.Pos;
import javafx.scene.Scene;
import javafx.scene.control.*;
import javafx.scene.layout.VBox;
import javafx.stage.Modality;
import javafx.stage.Stage;

import java.nio.charset.StandardCharsets;
import java.security.MessageDigest;

import static com.application.DB.Account_handler.*;
import static com.application.DB.Constants.*;
import static com.application.GUI.Panes.LogoBar.getLogin;

public class LoginPopup {

    public static PasswordField PASSWORD_TEXT_FIELD = new PasswordField();

    public static void login(){

        Stage window = new Stage();
        window.initModality(Modality.APPLICATION_MODAL);
        window.setTitle("Login window");

        Label userNameLabel = new Label("Username:");
        Label passwordLabel = new Label("Password:");

        TextField userNameTextField = new TextField();
        getPasswordTextField().clear();

        Button closeButton = new Button("Close");
        Button loginButton = new Button("Login");

        closeButton.setOnAction(event -> window.close());
        loginButton.setOnAction(event -> {

            try {
                getAccountInformation(userNameTextField.getText(), hashPassword(getPasswordTextField().getText()));
                    if(getLogin().getText().equals("Admin")) {
                        LoginPopup.adminPopup();
                        window.close();
                    }
            } catch (Exception e) {
                e.printStackTrace();
            }

            if(!getLogin().getText().equals("Login")){
                window.close();
            }

        });



        VBox layout = new VBox(10);
        layout.setAlignment(Pos.CENTER);
        layout.getChildren().addAll(userNameLabel, userNameTextField, passwordLabel, getPasswordTextField(), loginButton, closeButton);

        Scene scene = new Scene(layout, 500, 300);
        scene.getStylesheets().add(InputPopup.class.getResource("/com.application/CSS/styleSheet.css").toExternalForm());
        window.setScene(scene);
        window.showAndWait();
    }

    public static void adminPopup(){

        Stage window = new Stage();
        window.initModality(Modality.APPLICATION_MODAL);
        window.setTitle("Admin window");

        Label usernameLabel = new Label("Username: ");
        TextField usernameTextfield = new TextField();
        usernameTextfield.setText(getUserName());
        usernameTextfield.setEditable(false);

        Button addUser = new Button("Add User");
        Button deleteUser = new Button("Delete User");
        Button logout = new Button("Logout");
        Button close = new Button("Close");

        addUser.setOnAction(event -> {
            adminAddUser();
        });
        deleteUser.setOnAction(event -> {
            adminDeleteUser();
        });
        logout.setOnAction(event -> {
            logout();
            window.close();
        });
        close.setOnAction(event -> window.close());


        VBox layout = new VBox(10);
        layout.setAlignment(Pos.CENTER);
        layout.getChildren().addAll(usernameLabel, usernameTextfield, addUser, deleteUser, logout, close);

        Scene scene = new Scene(layout, 500, 300);
        scene.getStylesheets().add(InputPopup.class.getResource("/com.application/CSS/styleSheet.css").toExternalForm());
        window.setScene(scene);
        window.showAndWait();
    }

    public static void adminAddUser(){

        Stage window = new Stage();
        window.initModality(Modality.APPLICATION_MODAL);
        window.setTitle("Admin window");

        Label firstNameLabel = new Label("First Name: ");
        Label lastNameLabel = new Label("Last Name: ");
        Label phoneNoLabel = new Label("Phone No: ");
        Label usernameLabel = new Label("Username: ");
        Label passwordFirstLabel = new Label("Password : ");
        Label passwordSecondLabel = new Label("Password Repeat: ");
        Label isAdminLabel = new Label("Is Admin: ");

        TextField firstNameTextField = new TextField();
        TextField lastNameTextField = new TextField();
        TextField phoneNoTextField = new TextField();
        TextField usernameTextField = new TextField();
        PasswordField passwordFirstField = new PasswordField();
        PasswordField passwordSecondField = new PasswordField();
        CheckBox isAdminBox = new CheckBox();
        isAdminBox.setSelected(false);

        Button close = new Button("Close");
        Button addUser = new Button("Add User");

        close.setOnAction(event -> window.close());
        addUser.setOnAction(event -> {

            // If the passwords match each other, add the user, if not display an errormessage
            if(passwordFirstField.getText().contentEquals(passwordSecondField.getText())){

                // Hashing the password if they match.
                String hashedPassword = hashPassword(passwordFirstField.getText());

                // Tries to add the user
                try {
                    addUser(firstNameTextField.getText(), lastNameTextField.getText(), phoneNoTextField.getText(),
                            usernameTextField.getText(), hashedPassword, isAdminBox.isSelected());
                    NotificationPopUp.displayNotificationWindow("Successfully added user!");
                    window.close();
                } catch (Exception e) {
                    e.printStackTrace();
                }
            } else {
                NotificationPopUp.displayNotificationWindow("Passwords does not match!");
                passwordFirstField.clear();
                passwordSecondField.clear();
            }

        });


        VBox layout = new VBox(10);
        layout.setAlignment(Pos.CENTER);
        layout.getChildren().addAll(firstNameLabel, firstNameTextField, lastNameLabel, lastNameTextField,
                                    phoneNoLabel, phoneNoTextField, usernameLabel, usernameTextField,
                                    passwordFirstLabel, passwordFirstField, passwordSecondLabel, passwordSecondField,
                                    isAdminLabel, isAdminBox, addUser, close);

        Scene scene = new Scene(layout, 500, 600);
        scene.getStylesheets().add(InputPopup.class.getResource("/com.application/CSS/styleSheet.css").toExternalForm());
        window.setScene(scene);
        window.showAndWait();
    }

    public static void adminDeleteUser(){

        Stage window = new Stage();
        window.initModality(Modality.APPLICATION_MODAL);
        window.setTitle("Admin window");

        Label usernameLabel = new Label("Username: ");
        TextField usernameTextField = new TextField();

        Button close = new Button("Close");
        Button delete = new Button("Delete User");

        close.setOnAction(event -> window.close());
        delete.setOnAction(event -> {
            try {
                boolean results = deleteUser(usernameTextField.getText());
                if(results){
                    NotificationPopUp.displayNotificationWindow(usernameTextField.getText()+" was successfully deleted!");
                    window.close();
                } else {
                    NotificationPopUp.displayNotificationWindow("Could not find username: " + usernameTextField.getText());
                    usernameTextField.clear();
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        });

        VBox layout = new VBox(10);
        layout.setAlignment(Pos.CENTER);
        layout.getChildren().addAll(usernameLabel, usernameTextField, delete, close);

        Scene scene = new Scene(layout, 500, 300);
        scene.getStylesheets().add(InputPopup.class.getResource("/com.application/CSS/styleSheet.css").toExternalForm());
        window.setScene(scene);
        window.showAndWait();
    }

    public static void logout(){
        getLogin().setText("Login");
        setFirstName("");
        setLastName("");
        setIsAdmin(false);
        setPhoneNo("");
        setUserName("");
    }

    public static void userPopup(){

        Stage window = new Stage();
        window.initModality(Modality.APPLICATION_MODAL);
        window.setTitle("User window");


        Label nameLabel = new Label("Name: ");
        Label phoneNoLabel = new Label("Phone no: ");

        TextField nameTextfield = new TextField();
        TextField phoneNoTextField = new TextField();

        nameTextfield.setEditable(false);
        phoneNoTextField.setEditable(false);

        nameTextfield.setText(getFirstName() + " " + getLastName());
        phoneNoTextField.setText(getPhoneNo());


        Button close = new Button("Close");
        Button logout = new Button("Logout");
        close.setOnAction(event -> window.close());
        logout.setOnAction(event -> {
            logout();
            window.close();
        });

        VBox layout = new VBox(10);
        layout.setAlignment(Pos.CENTER);
        layout.getChildren().addAll(nameLabel, nameTextfield, phoneNoLabel, phoneNoTextField, logout, close);

        Scene scene = new Scene(layout, 500, 300);
        scene.getStylesheets().add(InputPopup.class.getResource("/com.application/CSS/styleSheet.css").toExternalForm());
        window.setScene(scene);
        window.showAndWait();
    }

    public static String hashPassword(String password){
        try {
            MessageDigest messageDigest = MessageDigest.getInstance("SHA-512");

            assert messageDigest != null;
            messageDigest.update(password.getBytes(StandardCharsets.UTF_8));

            byte[] hashedPassword = messageDigest.digest();

            StringBuilder hashedPasswordString = new StringBuilder();

            for (byte b: hashedPassword) {
                hashedPasswordString.append(String.format("%02x",b));
            }

            return hashedPasswordString.toString();

        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    public static PasswordField getPasswordTextField() {
        return PASSWORD_TEXT_FIELD;
    }
}
